﻿EnableExplicit

;- Information.
; Dieses Modul ist ein Testfenster für die Bildschirmkopie der Rigolgeräte.

;- Version.
; Status:   In Arbeit.
; Autor:    Peter Dreisiebner
; E-Mail:   web@dreisiebner.at
; Website:  http://peter.dreisiebner.at
; Geändert: 2017-03-19 - PureBasic v5.60 (x86).


;-
;- DeclareModule WindowStart. --------------------
;-
DeclareModule WindowScreencopy
   EnableExplicit
   
   Declare.i Open()

EndDeclareModule  ; WindowScreencopy.

;
;- Include.
;

;-
;- Module WindowScreencopy. --------------------
;-
Module WindowScreencopy
   EnableExplicit
   UseModule Common
   UseModule Macros
   UseModule Tool
   DebugOutWindowScreencopy

   ;- Konstanten.
   #iWindowWidth = 800
   #iWindowHeight = 590
   #sWindowTitle = "Bildschirmkopie"
   
   ;- Strukturen.
   ;- uWindow
   Structure uWindow
      iWindowNumber.i
      iButtonNumber.i
      iCheckboxNumber.i
      iTextNumber.i
      iListviewNumber.i
      iImageNumber.i
      ;
      iUsbTmcInstance.i  
      List DeviceList.UsbTmc::uDevice()
      fDeviceSelected.i
      sModel.s
      sSCPI.s
      iFreeImageNumber.i
      iImageCounter.i
      iEventCounter.i
      fCreateThread.i
   EndStructure
   
   ;- Variablen.
   Global NewMap mWindowMap.uWindow()
   Global miMutex.i
   
   ;- Declares.
   Declare Close(iWindowNumber.i)
   Declare UsbTmcSendRequestThreadData(iWindowNumber.i)
   ;-

   Procedure UsbTmcDeleteInstance(iWindowNumber.i)
      ; Ereignis wenn auf die Schaltfläche geklickt wurde.
      DebugView::ProcIn(iWindowNumber)
      Protected sKey.s, sError.String
      
      sKey = Str(iWindowNumber)
      LockMutex(miMutex)
      If FindMapElement(mWindowMap(), sKey)
         ; Vorhandene Instanz löschen.
         If mWindowMap()\iUsbTmcInstance
            If (Not UsbTmc::DeleteInstance(mWindowMap()\iUsbTmcInstance, sError))
               WindowLog::Error(CurrentProcedureName + "Die UsbTmc-Instanz konnte nicht gelöscht werden.")
               WindowLog::Error(CurrentProcedureName + sError\s)
            EndIf
            mWindowMap()\iUsbTmcInstance = #Null
            mWindowMap()\fDeviceSelected = #False
         EndIf
      EndIf
      UnlockMutex(miMutex)
      
      DebugView::ProcOut(iWindowNumber)
   EndProcedure


   Procedure UsbTmcThreadDataEvent()
      ; Ereignis wenn Daten an das USB-Gerät gesendet, oder Daten empfangen wurden.
      Protected sError.String, sKey.s, iWindowNumber.i, iCheckboxState.i, iImageNumber.i, fThreadFinished.i
      Protected iUsbTmc.i, iEvent.i, iEventData.i, uData.UsbTmc::uThreadData
      Protected sIDN.s
      
      iWindowNumber = EventWindow()
      DebugView::ProcIn(iWindowNumber)
      iUsbTmc = EventGadget()
      iEvent = EventType()
      iEventData = EventData()
      sKey = Str(iWindowNumber)
      
      LockMutex(miMutex)
      
      If Not (UsbTmc::#eCustomEventUsbTmcThreadData = Event())
         WindowLog::Error(CurrentProcedureName + "Ungültiges Ereignis empfangen.")
         
      ElseIf (Not FindMapElement(mWindowMap(), sKey))
         WindowLog::Error(CurrentProcedureName + "Das Fenster ist nicht in der mWindowMap. iWindowNumber = " + iWindowNumber)
         
      ElseIf Not (iUsbTmc = mWindowMap()\iUsbTmcInstance)
         WindowLog::Error(CurrentProcedureName + "Die UsbTmc-Instanz ist ungültig. iUsbTmc = " + iUsbTmc +
                          " - mWindowMap()\iUsbTmcInstance " + mWindowMap()\iUsbTmcInstance)
         
      Else
         Select iEvent
            Case UsbTmc::#eThreadEventSendProgress
               DebugView::Temp("Daten gesendet. Bytes = " + iEventData)
               
            Case UsbTmc::#eThreadEventSendFinished
               DebugView::Temp("Daten gesendet fertig.")
               
            Case UsbTmc::#eThreadEventRequestProgress
               DebugView::Temp("Daten empfangen. Bytes = " + iEventData)
               
            Case UsbTmc::#eThreadEventRequestFinished
               DebugView::Temp("Daten empfangen fertig.")
               
               If (Not UsbTmc::GetThreadData(iUsbTmc, uData, sError))
                  WindowLog::Error(CurrentProcedureName + "Die Threaddaten konnten nicht ermittelt werden.")
                  WindowLog::Error(CurrentProcedureName + sError\s)
                  
               Else
                  If Not (uData\iEventListenerWindowNumber = iWindowNumber)
                     WindowLog::Error(CurrentProcedureName + "Das Ereignis stimmt nicht mit dem Fenster überein.")
                     WindowLog::Error(CurrentProcedureName + "iWindowNumber = " + iWindowNumber + " - " +
                                      "uData\iEventListenerWindowNumber = " + uData\iEventListenerWindowNumber)
                     
                  Else
                     If (Not uData\fSendOk)
                        WindowLog::Error(CurrentProcedureName + "Fehler beim Senden: " + uData\sError)
                        
                     ElseIf (Not uData\fRequestOk)
                        WindowLog::Error(CurrentProcedureName + "Fehler beim Empfangen: " + uData\sError)
                        
                     ElseIf (Not uData\RequestData)
                        WindowLog::Error(CurrentProcedureName + "Der Zeiger auf die Daten ist Null. uData\RequestData = 0")
                        
                     Else
                        ; PIC18F-Test.
                        If (Left(mWindowMap()\sModel, 3) = "PIC")
                           sIDN = PeekS(uData\RequestData, uData\iRequestTransferred, #PB_Ascii)
                           WindowLog::Write("IDN: " + sIDN)
                           ; Zähler für die empfangenen IDNs aktualisieren.
                           mWindowMap()\iImageCounter + 1
                           SetGadgetText(mWindowMap()\iTextNumber, Str(mWindowMap()\iImageCounter))
                           
                           ; Picotest M3500A-Test.
                        ElseIf (Left(mWindowMap()\sModel, 3) = "M35")
                           sIDN = PeekS(uData\RequestData, uData\iRequestTransferred, #PB_Ascii)
                           WindowLog::Write("IDN: " + sIDN)
                           ; Zähler für die empfangenen IDNs aktualisieren.
                           mWindowMap()\iImageCounter + 1
                           SetGadgetText(mWindowMap()\iTextNumber, Str(mWindowMap()\iImageCounter))
                          
                        Else ; Rigol-Bitmap.
                           If mWindowMap()\iFreeImageNumber
                              If (Not IsImage(mWindowMap()\iFreeImageNumber))
                                 WindowLog::Error(CurrentProcedureName + "Das Image für die Freigabe ist ungültig. " +
                                                  "(mWindowMap()\iFreeImageNumber = $" + Hex(mWindowMap()\iFreeImageNumber))
                              Else
                                 FreeImage(mWindowMap()\iFreeImageNumber)
                              EndIf
                           EndIf
                           ; SCPI-Block ignorieren, darum die +-11.
                           iImageNumber = CatchImage(#PB_Any, uData\RequestData + 11, uData\iRequestTransferred - 11)
                           If (Not iImageNumber)
                              WindowLog::Error(CurrentProcedureName + "Die Daten konnten nicht in ein Bild umgewandelt werden.")
                              
                           Else
                              SetGadgetState(mWindowMap()\iImageNumber, ImageID(iImageNumber))
                              mWindowMap()\iFreeImageNumber = iImageNumber
                              ; Zähler für die empfangenen Bildschirmkopien aktualisieren.
                              mWindowMap()\iImageCounter + 1
                              SetGadgetText(mWindowMap()\iTextNumber, Str(mWindowMap()\iImageCounter))
                           EndIf
                        EndIf
                        FreeMemory(uData\RequestData)
                     EndIf
                  EndIf
               EndIf
               
            Case UsbTmc::#eThreadEventThreadFinished
               fThreadFinished = #True
               
            Default
               WindowLog::Error(CurrentProcedureName + "Ungültiges Ereignis empfangen. iEvent = " + iEvent)
         EndSelect
      EndIf
      
      UnlockMutex(miMutex)
      
      ; Die Checkbox abfragen für eine neue Bildschirmkopie.
      If fThreadFinished
         If (#PB_Checkbox_Checked = GetGadgetState(mWindowMap()\iCheckboxNumber))
            UsbTmcSendRequestThreadData(iWindowNumber)
         EndIf
      EndIf
      
      DebugView::ProcOut(iWindowNumber)
   EndProcedure
   

   Procedure UsbTmcSendRequestThreadData(iWindowNumber.i)
      DebugView::ProcIn(iWindowNumber)
      Protected fRequest.i, sKey.s, iUsbTmc.i, fCreateThread.i, sError.String
      Protected uData.UsbTmc::uThreadData
      
      LockMutex(miMutex)
      sKey = Str(iWindowNumber)
      If (Not FindMapElement(mWindowMap(), sKey))
         WindowLog::Error(CurrentProcedureName + "Das Fenster ist nicht in der mWindowMap. iWindowNumber = " + iWindowNumber)
         
      Else
         If (Not mWindowMap()\fDeviceSelected)
            WindowLog::Error(CurrentProcedureName + "Das Gerät ist nicht ausgewählt. mWindowMap()\fDeviceSelected = #False")
            
         Else
            iUsbTmc = mWindowMap()\iUsbTmcInstance
            If (Not iUsbTmc)
               WindowLog::Error(CurrentProcedureName + "Die UsbTmc-Instanz ist Null. mWindowMap()\iUsbTmcInstance = 0")
               
            Else
               With uData
                  \iUsbTmcInstance = iUsbTmc
                  \iEventListenerWindowNumber = iWindowNumber
                  \fSendData = #True
                  \sScpi = mWindowMap()\sSCPI
                  \SendData = #Null
                  \iSendDataLength = 0
                  \eSendMessageId = UsbTmc::#eMessageIdDevDepMsgOut
                  \fRequestData = #True
                  \RequestData = #Null
                  \eRequestMessageId = UsbTmc::#eMessageIdRequestDevDepMsgIn
                  \fWaitDataServiceRequest = #False
                  \fWaitDataStatusByte = #False
               EndWith
               fRequest = #True
            EndIf
         EndIf
      EndIf
      UnlockMutex(miMutex)
               
      If fRequest
         If (Not UsbTmc::SendRequestThreadData(iUsbTmc, uData, sError))
            WindowLog::Error(CurrentProcedureName + iUsbTmc + " Der Thread für die Kommunikation konnte nicht erstellt werden.")
            WindowLog::Error(CurrentProcedureName + sError\s)
            ;
            WindowLog::Write(CurrentProcedureName + iUsbTmc + " Beende Thread.")
            If (Not UsbTmc::SetThreadStatus(iUsbTmc, UsbTmc::#eThreadStatusEnd, sError))
               WindowLog::Error(CurrentProcedureName + iUsbTmc + " Der Thread konnte nicht beendet werden.")
               WindowLog::Error(CurrentProcedureName + sError\s)
               
            Else
               LockMutex(miMutex)
               If FindMapElement(mWindowMap(), sKey)
                  mWindowMap()\fCreateThread = Bool(Not mWindowMap()\fCreateThread)
                  fCreateThread = mWindowMap()\fCreateThread
                EndIf
               UnlockMutex(miMutex)
               
               If fCreateThread
                  UsbTmcSendRequestThreadData(iWindowNumber)
               EndIf
            EndIf 
         EndIf
      EndIf
   
      DebugView::ProcOut(iWindowNumber)
   EndProcedure
   
   
   Procedure.i EventCloseWindow()
      ; Ereignisprozedur wenn das Fenster geschlossen werden soll.
      Protected iWindowNumber.i
      
      iWindowNumber = EventWindow() 
      DebugView::ProcIn(iWindowNumber)
      Close(iWindowNumber)
      
      DebugView::ProcOut(iWindowNumber)
   EndProcedure


   Procedure EventButton()
      ; Ereignis wenn auf die Schaltfläche geklickt wurde.
      Protected fResult.i, iWindowNumber.i, iUsbTmc.i, sKey.s, iDeviceCount, sDevice.s, sError.String
      
      iWindowNumber = EventWindow()
      DebugView::ProcIn(iWindowNumber)
      sKey = Str(iWindowNumber)
      SetWindowTitle(iWindowNumber, #sWindowTitle)
      
      ; Vorhandene Instanz löschen.
      UsbTmcDeleteInstance(iWindowNumber)
      
      ; Neu Instanz erstellen.
      iUsbTmc = UsbTmc::NewInstance(#Null, sError)
      If (Not iUsbTmc)
         WindowLog::Error(CurrentProcedureName + "Die UsbTmc-Instanz konnte nicht erstellt werden.")
         WindowLog::Error(CurrentProcedureName + sError\s)
         
      Else
         fResult = #True
      EndIf
      
      If fResult
         LockMutex(miMutex)
         If FindMapElement(mWindowMap(), sKey)
            With mWindowMap()
               \iUsbTmcInstance = iUsbTmc
               
               ; Die Liste mit den vorhandenen Geräten füllen.
               DisableGadget(\iListviewNumber, #False)
               ClearGadgetItems(\iListviewNumber)
               If (Not UsbTmc::GetTmcDeviceList(iUsbTmc, \DeviceList(), @iDeviceCount, sError))
                  WindowLog::Error(CurrentProcedureName + "Die USBTMC-Geräteliste konnte nicht ermittelt werden.")
                  WindowLog::Error(CurrentProcedureName + sError\s)
                  
               Else
                  If (Not iDeviceCount)
                     AddGadgetItem(\iListviewNumber, -1, "Keine verfügbaren Geräte vorhanden.")
                     SetGadgetItemData(\iListviewNumber, CountGadgetItems(\iListviewNumber) - 1, #Null)
                     
                  Else
                     ForEach \DeviceList()
                        sDevice = Hex(\DeviceList()\iVendorId) + "/" + Hex(\DeviceList()\iProductId) + " " +
                                  \DeviceList()\sManufacturer + ", " + \DeviceList()\sProduct + ", " + \DeviceList()\sSerialnumber
                        AddGadgetItem(\iListviewNumber, -1, sDevice)
                        SetGadgetItemData(\iListviewNumber, CountGadgetItems(\iListviewNumber) - 1, \DeviceList())
                     Next
                  EndIf
               EndIf
            EndWith
         EndIf
         UnlockMutex(miMutex)
      EndIf
      
      DebugView::ProcOut(iWindowNumber)
   EndProcedure


   Procedure EventCheckBox()
      ; Ereignis wenn auf die Checkbox geklickt wurde.
      Protected iWindowNumber.i, iNumber.i, iState.i, sKey.s, *sError.String
      
      iWindowNumber = EventWindow()
      DebugView::ProcIn(iWindowNumber)
      iNumber = EventGadget()
      iState = GetGadgetState(iNumber)
      
      If (iState = #PB_Checkbox_Checked)
         UsbTmcSendRequestThreadData(iWindowNumber)
      Else
         ; Als Test für das Abbrechen der Kommunikation.
         LockMutex(miMutex)
         sKey = Str(iWindowNumber)
         If (Not FindMapElement(mWindowMap(), sKey))
            WindowLog::Error(CurrentProcedureName + "Das Fenster ist nicht in der mWindowMap. iWindowNumber = " + iWindowNumber)
            
         Else
            If (Not mWindowMap()\fDeviceSelected)
               WindowLog::Error(CurrentProcedureName + "Das Gerät ist nicht ausgewählt. mWindowMap()\fDeviceSelected = #False")
               
            Else
               If (Not mWindowMap()\iUsbTmcInstance)
                  WindowLog::Error(CurrentProcedureName + "Die UsbTmc-Instanz ist Null. mWindowMap()\iUsbTmcInstance = 0")
                  
               Else
                  If (Not UsbTmc::SetThreadStatus(mWindowMap()\iUsbTmcInstance, UsbTmc::#eThreadStatusCancel, *sError))
                     WindowLog::Error(CurrentProcedureName + "Der Thread konnte nich abgebrochen werden.")
                  EndIf
               EndIf
            EndIf
         EndIf
         UnlockMutex(miMutex)
      EndIf
      
      DebugView::ProcOut(iWindowNumber)
   EndProcedure


   Procedure EventListviewLeftClick()
      ; Ereignis wenn ein Eintrag in dem ListViewGadget ausgewählt wurde.
      Protected sDevice.s, sKey.s, sError.String
      Protected iWindowNumber.i, iGadgetNumber.i, iItem.i, iUsbTmc.i, iDevice.i, fRequest.i
      
      iWindowNumber = EventWindow()
      DebugView::ProcIn(iWindowNumber)
      iGadgetNumber = EventGadget()
      iItem = GetGadgetState(iGadgetNumber)
      If (iItem > -1)
         iDevice = GetGadgetItemData(iGadgetNumber, iItem)
         If iDevice
            
            LockMutex(miMutex)
            sKey = Str(iWindowNumber)
            If FindMapElement(mWindowMap(), sKey)
               With mWindowMap()
                  iUsbTmc = \iUsbTmcInstance
                  \iEventCounter = #Null
                  
                  ForEach \DeviceList()
                     If (iDevice = \DeviceList())
                        If (Not UsbTmc::SetTmcDevice(iUsbTmc, \DeviceList(), sError))
                           WindowLog::Error(CurrentProcedureName + "Das USB-Gerät konnte nicht ausgewählt werden.")
                           WindowLog::Error(CurrentProcedureName + sError\s)
                           \sModel = #Empty$
                          
                        Else
                           \fDeviceSelected = #True
                           \sModel = \DeviceList()\sProduct
                           sDevice = Hex(\DeviceList()\iVendorId) + "/" + Hex(\DeviceList()\iProductId) + " " +
                                     \DeviceList()\sManufacturer + ", " + \DeviceList()\sProduct + ", " +
                                     \DeviceList()\sSerialnumber
                           WindowLog::Write(CurrentProcedureName + "Das USB-Gerät wurde ausgewählt.")
                           WindowLog::Write(CurrentProcedureName + sDevice)
                           DisableGadget(iGadgetNumber, #True)
                           SetWindowTitle(iWindowNumber, #sWindowTitle + " - " + \DeviceList()\sProduct)
                           
                           ; SCPI-Kommando für die Bildschirmkopie der Rigolgeräte.
                           sDevice = Left(\DeviceList()\sProduct, 3)
                           If (sDevice = "DSA")
                              mWindowMap()\sSCPI = "*SRE 16;:PRIV:SNAP? BMP"
                           ElseIf (sDevice = "DS1")
                              mWindowMap()\sSCPI = "*SRE 16;:DISP:DATA?"
                           ElseIf (sDevice = "MSO")
                              mWindowMap()\sSCPI = "*SRE 16;:DISP:DATA?"
                           ElseIf (sDevice = "DP8")
                              mWindowMap()\sSCPI = "*SRE 16;:SYSTem:PRINT? BMP"
                           ElseIf (sDevice = "DG4")
                              mWindowMap()\sSCPI = "*SRE 16;:HCOPy:SDUMp:DATA?"
                           ; PIC18F4550-Test.   
                           ElseIf (sDevice = "PIC")
                              mWindowMap()\sSCPI = "*IDN?"
                           ; Picotest M3500A-Test.   
                           ElseIf (sDevice = "M35")
                              mWindowMap()\sSCPI = "*IDN?"
                           EndIf
                           
                           ; TODO - Multiple Transfers testen.
;                            UsbTmc::SetTransferSplitDataLength(iUsbTmc, 5, sError)   ; Test für Multiple Transfers, UsbTmc::SendData().
;                            UsbTmc::SetRequestBufferLength(iUsbTmc, $4000, sError)  ; 16 KB.

                           fRequest = #True
                           Break ; ForEach \DeviceList().
                        EndIf
                     EndIf
                  Next
               EndWith
            EndIf
            UnlockMutex(miMutex)
            
            If fRequest
               UsbTmcSendRequestThreadData(iWindowNumber)
            EndIf
         EndIf
      EndIf
      
      DebugView::ProcOut(iWindowNumber)
   EndProcedure

   ;-

   Procedure.i Open()
      ; Das Fenster laden und anzeigen.
      DebugView::ProcIn()
      Protected iWindowNumber.i, iGadgetNumber.i

      iWindowNumber = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, #iWindowWidth, #iWindowHeight, #sWindowTitle,
                                  #PB_Window_MinimizeGadget)
      If (Not iWindowNumber)
         DebugView::Error("Das Fenster >" + #PB_Compiler_Module + "< konnte nicht erstellt werden.", iWindowNumber)

      Else
         If (Not miMutex)
            miMutex = CreateMutex()
         EndIf
         LockMutex(miMutex)
         
         AddMapElement(mWindowMap(), Str(iWindowNumber))
         
         BindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), iWindowNumber)
         BindEvent(UsbTmc::#eCustomEventUsbTmcThreadData, @UsbTmcThreadDataEvent(), iWindowNumber)
         Tool::WindowReduceFlicker(iWindowNumber)
         
         iGadgetNumber = ButtonGadget(#PB_Any, 10, 10, 80, 25, "Neu")
         If iGadgetNumber
            mWindowMap()\iButtonNumber = iGadgetNumber
            BindGadgetEvent(iGadgetNumber, @EventButton())
            SetActiveGadget(iGadgetNumber)
         EndIf
         
         iGadgetNumber = CheckBoxGadget(#PB_Any, 10, 45, 90, 25, "Endlos")
         If iGadgetNumber
            mWindowMap()\iCheckboxNumber = iGadgetNumber
            SetGadgetState(iGadgetNumber, #PB_Checkbox_Checked)
            BindGadgetEvent(iGadgetNumber, @EventCheckBox())
         EndIf
         
         iGadgetNumber = TextGadget(#PB_Any, 10, 80, 90, 20, "0")
         If iGadgetNumber
            mWindowMap()\iTextNumber = iGadgetNumber
         EndIf
         
         iGadgetNumber = ListViewGadget(#PB_Any, 100, 10, 690, 90)
         If iGadgetNumber
            mWindowMap()\iListviewNumber = iGadgetNumber
            BindGadgetEvent(iGadgetNumber, @EventListviewLeftClick(), #PB_EventType_LeftClick)
         EndIf
         
         iGadgetNumber = ImageGadget(#PB_Any, 0, 110, 800, 480, #Null)
         mWindowMap()\iImageNumber = iGadgetNumber
         
         UnlockMutex(miMutex)
      EndIf

      DebugView::ProcOut(iWindowNumber)
      ProcedureReturn iWindowNumber
   EndProcedure


   Procedure Close(iWindowNumber.i)
      ; Schließt das Fenster.
      DebugView::ProcIn(iWindowNumber)

      If iWindowNumber
;          UnbindGadgetEvent(iGadgetNumber, @EventButton())
;          UnbindGadgetEvent(iGadgetNumber, @EventCheckBox())
;          UnbindGadgetEvent(iGadgetNumber, @EventListviewLeftClick(), #PB_EventType_LeftClick)
         UnbindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), iWindowNumber)
         UnbindEvent(UsbTmc::#eCustomEventUsbTmcThreadData, @UsbTmcThreadDataEvent(), iWindowNumber)
         
         ; Vorhandene UsbTmc-Instanz löschen.
         UsbTmcDeleteInstance(iWindowNumber)
         
         CloseWindow(iWindowNumber)
      EndIf

      DebugView::ProcOut(iWindowNumber)
      ProcedureReturn
   EndProcedure


EndModule   ; WindowScreencopy.
