﻿EnableExplicit

;- -- Information.
; Dieses Modul enthält die Prozeduren für die LibUsb 1.0.21.
;
; http://libusb.info/
;
; Die Dokumentation in den Prozeduren stammt aus dem Sourcecode von LibUsb 1.0.21.

;- -- Version 01.
; Status:   In Arbeit, aber was vorhanden ist funktioniert, ansonst siehe die 'Issues'.
;           Das Modul und die LibUsb sind sicher für die Verwendung in Threads.
; Autor:    Peter Dreisiebner
; E-Mail:   web@dreisiebner.at
; Website:  http://peter.dreisiebner.at
; Geändert: 2017-10-27 - PureBasic v5.61 (x86).
;
; 2017-10-27
;  - GetStringLanguageIdList hinzugefügt und libusb_get_string_descriptor angepasst.
; 2017-10-19
;  - libusb_get_string_descriptor hinzugefügt.
; 2017-10-15
;  - libusb_clear_halt hinzugefügt.

;-
;- -- DeclareModule LibUsb. --------------------
;
DeclareModule LibUsb
   EnableExplicit
   
   ;-
   ;- -- Enumeration LibUsb.
   ;- libusb_bos_type.
   Enumeration
      #eLIBUSB_BT_WIRELESS_USB_DEVICE_CAPABILITY = 1
      #eLIBUSB_BT_USB_2_0_EXTENSION = 2
      #eLIBUSB_BT_SS_USB_DEVICE_CAPABILITY = 3
      #eLIBUSB_BT_CONTAINER_ID = 4
   EndEnumeration

   ;- libusb_capability.
   Enumeration
      #eLIBUSB_CAP_HAS_CAPABILITY = 0
      #eLIBUSB_CAP_HAS_HOTPLUG = 1
      #eLIBUSB_CAP_HAS_HID_ACCESS = $0100
      #eLIBUSB_CAP_SUPPORTS_DETACH_KERNEL_DRIVER = $0101
   EndEnumeration

   ;- libusb_class_code.
   Enumeration
      #eLIBUSB_CLASS_PER_INTERFACE = 0
      #eLIBUSB_CLASS_AUDIO = 1
      #eLIBUSB_CLASS_COMM = 2
      #eLIBUSB_CLASS_HID = 3
      #eLIBUSB_CLASS_PHYSICAL = 5
      #eLIBUSB_CLASS_PTP = 6
      #eLIBUSB_CLASS_IMAGE = 6
      #eLIBUSB_CLASS_PRINTER = 7
      #eLIBUSB_CLASS_MASS_STORAGE = 8
      #eLIBUSB_CLASS_HUB = 9
      #eLIBUSB_CLASS_DATA = 10
      #eLIBUSB_CLASS_SMART_CARD = $0B
      #eLIBUSB_CLASS_CONTENT_SECURITY = $0D
      #eLIBUSB_CLASS_VIDEO = $0E
      #eLIBUSB_CLASS_PERSONAL_HEALTHCARE = $0F
      #eLIBUSB_CLASS_DIAGNOSTIC_DEVICE = $DC
      #eLIBUSB_CLASS_WIRELESS = $E0
      #eLIBUSB_CLASS_APPLICATION = $FE
      #eLIBUSB_CLASS_VENDOR_SPEC = $FF
   EndEnumeration

   ;- libusb_descriptor_type.
   Enumeration
      #eLIBUSB_DT_DEVICE = 1
      #eLIBUSB_DT_CONFIG = 2
      #eLIBUSB_DT_STRING = 3
      #eLIBUSB_DT_INTERFACE = 4
      #eLIBUSB_DT_ENDPOINT = 5
      #eLIBUSB_DT_BOS = $0F
      #eLIBUSB_DT_DEVICE_CAPABILITY = $10
      #eLIBUSB_DT_HID = $21
      #eLIBUSB_DT_REPORT = $22
      #eLIBUSB_DT_PHYSICAL = $23
      #eLIBUSB_DT_HUB = $29
      #eLIBUSB_DT_SUPERSPEED_HUB = $2A
      #eLIBUSB_DT_SS_ENDPOINT_COMPANION = $30
   EndEnumeration

   ;- libusb_endpoint_direction.
   Enumeration
      #eLIBUSB_ENDPOINT_IN = $80
      #eLIBUSB_ENDPOINT_OUT = 0
   EndEnumeration

   ;- libusb_error.
   Enumeration
      #eLIBUSB_SUCCESS = 0
      #eLIBUSB_ERROR_IO = -1
      #eLIBUSB_ERROR_INVALID_PARAM = -2
      #eLIBUSB_ERROR_ACCESS = -3
      #eLIBUSB_ERROR_NO_DEVICE = -4
      #eLIBUSB_ERROR_NOT_FOUND = -5
      #eLIBUSB_ERROR_BUSY = -6
      #eLIBUSB_ERROR_TIMEOUT = -7
      #eLIBUSB_ERROR_OVERFLOW = -8
      #eLIBUSB_ERROR_PIPE = -9
      #eLIBUSB_ERROR_INTERRUPTED = -10
      #eLIBUSB_ERROR_NO_MEM = -11
      #eLIBUSB_ERROR_NOT_SUPPORTED = -12
      #eLIBUSB_ERROR_OTHER = -99
   EndEnumeration

   ;- libusb_hotplug_event.
   Enumeration
      #eLIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED = 1
      #eLIBUSB_HOTPLUG_EVENT_DEVICE_LEFT = 2
   EndEnumeration

   ;- libusb_hotplug_flag.
   Enumeration
      #eLIBUSB_HOTPLUG_NO_FLAGS = 0
      #eLIBUSB_HOTPLUG_ENUMERATE = 1
   EndEnumeration

   ;- libusb_iso_sync_type.
   Enumeration
      #eLIBUSB_ISO_SYNC_TYPE_NONE = 0
      #eLIBUSB_ISO_SYNC_TYPE_ASYNC = 1
      #eLIBUSB_ISO_SYNC_TYPE_ADAPTIVE = 2
      #eLIBUSB_ISO_SYNC_TYPE_SYNC = 3
   EndEnumeration

   ;- libusb_iso_usage_type.
   Enumeration
      #eLIBUSB_ISO_USAGE_TYPE_DATA = 0
      #eLIBUSB_ISO_USAGE_TYPE_FEEDBACK = 1
      #eLIBUSB_ISO_USAGE_TYPE_IMPLICIT = 2
   EndEnumeration

   ;- libusb_log_level.
   Enumeration
      #eLIBUSB_LOG_LEVEL_NONE = 0
      #eLIBUSB_LOG_LEVEL_ERROR = 1
      #eLIBUSB_LOG_LEVEL_WARNING = 2
      #eLIBUSB_LOG_LEVEL_INFO = 3
      #eLIBUSB_LOG_LEVEL_DEBUG = 4
   EndEnumeration

   ;- libusb_request_direction. Eigene Enumeration für Control Transfer.
   Enumeration
      #eLIBUSB_REQUEST_HOSTTODEVICE = 0
      #eLIBUSB_REQUEST_DEVICETOHOST = $80 ; Bit 7 ist gesetzt.
   EndEnumeration

   ;- libusb_request_recipient.
   Enumeration
      #eLIBUSB_RECIPIENT_DEVICE = 0
      #eLIBUSB_RECIPIENT_INTERFACE = 1
      #eLIBUSB_RECIPIENT_ENDPOINT = 2
      #eLIBUSB_RECIPIENT_OTHER = 3
   EndEnumeration

   ;- libusb_request_type.
   Enumeration
      #eLIBUSB_REQUEST_TYPE_STANDARD = $0 << 5
      #eLIBUSB_REQUEST_TYPE_CLASS = $01 << 5
      #eLIBUSB_REQUEST_TYPE_VENDOR = $02 << 5
      #eLIBUSB_REQUEST_TYPE_RESERVED = $03 << 5
   EndEnumeration

   ;- libusb_speed.
   Enumeration
      #eLIBUSB_SPEED_UNKNOWN = 0
      #eLIBUSB_SPEED_LOW = 1
      #eLIBUSB_SPEED_FULL = 2
      #eLIBUSB_SPEED_HIGH = 3
      #eLIBUSB_SPEED_SUPER = 4
   EndEnumeration

   ;- libusb_ss_usb_device_capability_attributes.
   Enumeration
      #eLIBUSB_BM_LTM_SUPPORT = 2
   EndEnumeration

   ;- libusb_standard_request.
   Enumeration
      #eLIBUSB_REQUEST_GET_STATUS = 0
      #eLIBUSB_REQUEST_CLEAR_FEATURE = 1
      #eLIBUSB_REQUEST_SET_FEATURE = 3
      #eLIBUSB_REQUEST_SET_ADDRESS = 5
      #eLIBUSB_REQUEST_GET_DESCRIPTOR = 6
      #eLIBUSB_REQUEST_SET_DESCRIPTOR = 7
      #eLIBUSB_REQUEST_GET_CONFIGURATION = 8
      #eLIBUSB_REQUEST_SET_CONFIGURATION = 9
      #eLIBUSB_REQUEST_GET_INTERFACE = $0A
      #eLIBUSB_REQUEST_SET_INTERFACE = $0B
      #eLIBUSB_REQUEST_SYNCH_FRAME = $0C
      #eLIBUSB_REQUEST_SET_SEL = $30
      #eLIBUSB_SET_ISOCH_DELAY = $31
   EndEnumeration

   ;- libusb_supported_speed.
   Enumeration
      #eLIBUSB_LOW_SPEED_OPERATION = 1
      #eLIBUSB_FULL_SPEED_OPERATION = 2
      #eLIBUSB_HIGH_SPEED_OPERATION = 4
      #eLIBUSB_SUPER_SPEED_OPERATION = 8
   EndEnumeration

   ;- libusb_transfer_flags.
   Enumeration
      #eLIBUSB_TRANSFER_SHORT_NOT_OK = 1 << 0
      #eLIBUSB_TRANSFER_FREE_BUFFER = 1 << 1
      #eLIBUSB_TRANSFER_FREE_TRANSFER = 1 << 2
      #eLIBUSB_TRANSFER_ADD_ZERO_PACKET = 1 << 3
   EndEnumeration

   ;- libusb_transfer_status.
   Enumeration
      #eLIBUSB_TRANSFER_COMPLETED = 0
      #eLIBUSB_TRANSFER_ERROR = 1
      #eLIBUSB_TRANSFER_TIMED_OUT = 2
      #eLIBUSB_TRANSFER_CANCELLED = 3
      #eLIBUSB_TRANSFER_STALL = 4
      #eLIBUSB_TRANSFER_NO_DEVICE = 5
      #eLIBUSB_TRANSFER_OVERFLOW = 6
   EndEnumeration

   ;- libusb_transfer_type.
   Enumeration
      #eLIBUSB_TRANSFER_TYPE_CONTROL = 0
      #eLIBUSB_TRANSFER_TYPE_ISOCHRONOUS = 1
      #eLIBUSB_TRANSFER_TYPE_BULK = 2
      #eLIBUSB_TRANSFER_TYPE_INTERRUPT = 3
      #eLIBUSB_TRANSFER_TYPE_BULK_STREAM = 4
   EndEnumeration

   ;- libusb_usb_2_0_extension_attributes.
   Enumeration
      #eLIBUSB_BM_LPM_SUPPORT = 2
   EndEnumeration

   ;-
   ;- -- Strukturen LibUsb.
   #iLibUsbStructureAlignment = #PB_Structure_AlignC

   ;- list_head.
   Structure uListHead Align #iLibUsbStructureAlignment
      *uPrev.uListHead
      *uNext.uListHead
   EndStructure

   ; TODO - libusb_bos_descriptor. UNGETESTET!
   ;- libusb_bos_descriptor.
   Structure uBosDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iTotalLength.u
      iNumDeviceCaps.a
      *uDevCapability.uBosDevCapabilityDescriptor
   EndStructure

   ; TODO - libusb_bos_dev_capability_descriptor. UNGETESTET!
   ;- libusb_bos_dev_capability_descriptor.
   Structure uBosDevCapabilityDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iDevCapabilityType.a
      iDevCapabilityData.a[1]
   EndStructure

   ;- libusb_config_descriptor
   Structure uConfigDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iTotalLength.u
      iNumInterfaces.a
      iConfigurationValue.a
      iConfiguration.a
      iAttributes.a
      iMaxPower.a
      *uInterface.uInterface
      *aExtra.Ascii
      iExtraLength.i
   EndStructure

   ; TODO - libusb_container_id_descriptor. UNGETESTET!
   ;- libusb_container_id_descriptor.
   Structure uContainerIdDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iDevCapabilityType.a
      iReserved.a
      iContainerId.a[16]
   EndStructure

   ;- libusb_context. NICHT VERWENDEN!
   ; Hinweis: Es wird nur ein Zeiger auf die Struktur von der DLL verwendet.
   ; Die Struktur ändert sich zwischen den libusb-Versionen und ist nur für Testzwecke gedacht.
   Structure uContext Align #iLibUsbStructureAlignment
      iDebug.i
      iDebugFixed.i
      iEventPipe.i[2]
      uUsbDevs.uListHead
      iUsbDevsLock.i
      uOpenDevs.uListHead
      iOpenDevsLock.i
      uHotplugCbs.uListHead
      iHotplugCbsLock.i
      uFlyingTransfers.uListHead
      iFlyingTransfersLock.i
      *FdAddedCb
      *FdRemovedCb
      *FdCbUserData
      iEventsLock.i
      iEventHandlerActive.i
      iEventHandlingKey.i
      iEventWaitersLock.i
      iEventWaitersCond.i
      iEventDataLock.i
      iEventFlags.i
      iDeviceClose.i
      uPollFds.uListHead
      *PollFds
      iPollFdsCnt.i
      uHotplugMsgs.uListHead
      uCompletedTransfers.uListHead
      iTimerFd.i  ; Used for timeout handling, if supported by OS. Ist nicht immer vorhanden, hängt vom System ab!
      uList.uListHead
   EndStructure

   ; TODO - libusb_control_setup. UNGETESTET!
   ;- libusb_control_setup.
   Structure uControlSetup Align #iLibUsbStructureAlignment
      iRequestType.a
      iRequest.a
      iValue.u
      iIndex.u
      iLength.u
   EndStructure

   ;- libusb_device_descriptor.
   Structure uDeviceDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iBcdUsb.u
      iDeviceClass.a
      iDeviceSubClass.a
      iDeviceProtocol.a
      iMaxPacketSize0.a
      iVendorId.u
      iProductId.u
      iBcdDevice.u
      iManufacturer.a
      iProduct.a
      iSerialNumber.a
      iNumConfigurations.a
   EndStructure

   ;- libusb_device
   Structure uDevice Align #iLibUsbStructureAlignment
      iLock.i
      iRefCnt.i
      *uContext.uContext
      iBusNumber.a
      iPortNumber.a
      *uParentDev.uDevice
      iDeviceAddress.a
      iNumConfigurations.a
      eSpeed.i ; Enthält Werte aus der "libusb_speed" Aufzählung.
      uList.uListHead
      iSessionData.l
      uDeviceDescriptor.uDeviceDescriptor
      iAttached.i
      iOSpriv.a[1]
   EndStructure

   ;- libusb_device_handle.
   Structure uDeviceHandle Align #iLibUsbStructureAlignment
      iLock.i
      iClaimedInterfaces.l
      uList.uListHead
      *uDev.uDevice
      iAutoDetachKernelDriver.i
      iOSpriv.a[1]
   EndStructure

   ;- libusb_endpoint_descriptor.
   Structure uEndpointDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iEndpointAddress.a
      iAttributes.a
      iMaxPacketSize.u
      iInterval.a
      iRefresh.a
      iSynchAddress.a
      *aExtra.Ascii
      iExtraLength.i
   EndStructure

   ;- libusb_interface.
   Structure uInterface Align #iLibUsbStructureAlignment
      *uAltSetting.uInterfaceDescriptor
      iNumAltSetting.i
   EndStructure

   ;- libusb_interface_descriptor.
   Structure uInterfaceDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iInterfaceNumber.a
      iAlternateSetting.a
      iNumEndpoints.a
      iInterfaceClass.a
      iInterfaceSubClass.a
      iInterfaceProtocol.a
      iInterface.a
      *uEndpoint.uEndpointDescriptor
      *aExtra.Ascii
      iExtraLength.i
   EndStructure

   ; TODO - libusb_iso_packet_descriptor. UNGETESTET!
   ;- libusb_iso_packet_descriptor.
   Structure uIsoPacketDescriptor Align #iLibUsbStructureAlignment
      iLength.i
      iActualLength.i
      eTransferStatus.i ; Enthält Werte aus der "libusb_transfer_status" Aufzählung.
   EndStructure

   ; TODO - libusb_pollfd. UNGETESTET!
   ;- libusb_pollfd.
   Structure uPollfd Align #iLibUsbStructureAlignment
      iFd.i
      iEvents.w
   EndStructure

   ; TODO - libusb_ss_endpoint_companion_descriptor. UNGETESTET!
   ;- libusb_ss_endpoint_companion_descriptor.
   Structure uSsEndpointCompanionDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iMaxBurst.a
      iAttributes.a
      iBytesPerInterval.u
   EndStructure

   ; TODO - libusb_ss_usb_device_capability_descriptor. UNGETESTET!
   ;- libusb_ss_usb_device_capability_descriptor.
   Structure uSsUsbDeviceCapabilityDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iDevCapabilityType.a
      iAttributes.a
      iSpeedSupported.u
      iFunctionalitySupport.a
      iU1DevExitLat.a
      iU2DevExitLat.u
   EndStructure

   ;- libusb_string_descriptor. Diese Struktur ist keine offizielle LibUsb-Struktur!
   Structure uStringDescriptor Align 1
      iLength.a
      iDescriptorType.a
      StructureUnion
         iCountryCode.u[0]
         uChars.u[0]
      EndStructureUnion
   EndStructure

   ; TODO - libusb_transfer. UNGETESTET!
   ;- libusb_transfer.
   Structure uTransfer Align #iLibUsbStructureAlignment
      *uDeviceHandle.uDeviceHandle
      iFlags.a
      iEndpoint.a
      iType.a
      iTimeout.i
      eStatus.i ; Enthält Werte aus der "libusb_transfer_status" Aufzählung.
      iLength.i
      iActualLength.i
      *TransferCallback
      *UserData
      *aBuffer.Ascii
      iNumIsoPackets.i
      uIsoPacketDescriptor.uIsoPacketDescriptor[1]
   EndStructure

   ; TODO - libusb_usb_2_0_extension_descriptor. UNGETESTET!
   ;- libusb_usb_2_0_extension_descriptor.
   Structure uUsb20ExtensionDescriptor Align #iLibUsbStructureAlignment
      iLength.a
      iDescriptorType.a
      iDevCapabilityType.a
      iAttributes.l
   EndStructure

   ;- libusb_version.
   Structure uVersion Align #iLibUsbStructureAlignment
      iMajor.u
      iMinor.u
      iMicro.u
      iNano.u
      *aReleaseCandidate.Ascii
      *aDescribe.Ascii
   EndStructure

   ; TODO - timeval. UNGETESTET!
   ;- timeval.
   ; Diese Struktur stammt nicht aus dem LibUsb-Quellcode.
   Structure uTimeval Align #iLibUsbStructureAlignment
      iSec.i      ; This represents the number of whole seconds of elapsed time.
      iUSec.i     ; This is the rest of the elapsed time (a fraction of a second), represented as the number of microseconds.
                  ; It is always less than one million.
   EndStructure

   ;-
   ;- -- Declares.
   Declare.i BulkTransfer(*uDeviceHandle, iEndpoint.i, *Data, iLength.i, *iTransferred.Integer, iTimeout.i, *sError.String)
   Declare.i ClaimInterface(*uDeviceHandle, iInterfaceNumber.i, *sError.String)
   Declare.i ClearHalt(*uDeviceHandle, iEndpoint.i, *sError.String)
   Declare Close(*uDeviceHandle)
   Declare.i ControlTransfer(*uDeviceHandle, iRequestType.a, iRequest.a, iValue.u, iIndex.u, *Data, *iLength.Integer,
                             iTimeout.i, *sError.String)
   Declare.s ErrorDescription(eErrorCode.i)
   Declare.s ErrorName(eErrorCode.i)
   Declare Exit(*uContext)
   Declare FreeConfigDescriptor(*uConfigDescriptor)
   Declare FreeDeviceList(*uDeviceList, fUnrefDevices.i = #True)
   Declare.i GetActiveConfigDescriptor(*uDevice, *uConfigDescriptor, *sError.String)
   Declare.i GetBusNumber(*uDevice, *iBusNumber.Integer, *sError.String)
   Declare.i GetConfigDescriptor(*uDevice, iIndex.i, *uConfigDescriptor, *sError.String)
   Declare.i GetDevice(*uDeviceHandle, *uDevice.Integer, *sError.String)
   Declare.i GetDeviceDescriptor(*uDevice, *uDeviceDescriptor, *sError.String)
   Declare.i GetDeviceList(*uContext, List DeviceList.uDevice(), *iDeviceCount.Integer, *uFreeDeviceList.Integer, *sError.String)
   Declare.i GetDeviceSpeed(*uDevice, *eSpeed.Integer, *sError.String)
   Declare.i GetPortNumber(*uDevice, *iPortNumber.Integer, *sError.String)
   Declare.i GetStringLanguageList(*uDeviceHandle, List LanguageIdList.i(), *sError.String)
   Declare.i GetStringDescriptor(*uDeviceHandle, iStringIndex.i, iLanguageId.i, *sString.String, *sError.String)
   Declare.i GetStringDescriptorAscii(*uDeviceHandle, iStringIndex.i, *sString.String, *sError.String)
   Declare.i GetVersion(*sVersion.String, *sDescription.String, *sError.String)
   Declare.i HasCapability(eCapability.i)
   Declare.i Init(*uContext, *sError.String)
   Declare.i InterruptTransfer(*uDeviceHandle, iEndpoint.i, *Data, iLength.i, *iTransferred.Integer, iTimeout.i, *sError.String)
   Declare.i Open(*uDevice, *uDeviceHandle, *sError.String)
   Declare.i ReleaseInterface(*uDeviceHandle, iInterfaceNumber.i, *sError.String)
   Declare.i SetConfiguration(*uDeviceHandle, iConfiguration.i, *sError.String)
   Declare SetDebug(*uContext, eLevel.i)
   Declare.i SetLocale(sLocale.s, *sError.String)
   Declare.s StrError(eErrorCode.i)

EndDeclareModule  ; LibUsb.


;-
;- -- Module LibUsb. --------------------
;
Module LibUsb
   EnableExplicit
   UseModule Common
   UseModule Macros
   DebugOutLibUsb

   ;-
   ;- -- Macros.
   
   ;-
   ;- -- Konstanten.
   
   ;-
   ;- -- Stukturen.
   Structure uDeviceList
      *uDevice.uDevice[0]
   EndStructure
   
   ;-
   ;- -- Variablen.

   ;-
   ;- -- Konstanten LibUsb.
   #LIBUSB_DT_DEVICE_SIZE = 18
   #LIBUSB_DT_CONFIG_SIZE = 9
   #LIBUSB_DT_INTERFACE_SIZE = 9
   #LIBUSB_DT_ENDPOINT_SIZE = 7
   #LIBUSB_DT_ENDPOINT_AUDIO_SIZE = 9
   #LIBUSB_DT_HUB_NONVAR_SIZE = 7
   #LIBUSB_DT_SS_ENDPOINT_COMPANION_SIZE = 6
   #LIBUSB_DT_BOS_SIZE = 5
   #LIBUSB_DT_DEVICE_CAPABILITY_SIZE = 3

   #LIBUSB_BT_USB_2_0_EXTENSION_SIZE = 7
   #LIBUSB_BT_SS_USB_DEVICE_CAPABILITY_SIZE = 10
   #LIBUSB_BT_CONTAINER_ID_SIZE = 20
   #LIBUSB_DT_BOS_MAX_SIZE = #LIBUSB_DT_BOS_SIZE +
                             #LIBUSB_BT_USB_2_0_EXTENSION_SIZE +
                             #LIBUSB_BT_SS_USB_DEVICE_CAPABILITY_SIZE +
                             #LIBUSB_BT_CONTAINER_ID_SIZE

   #LIBUSB_ENDPOINT_ADDRESS_MASK = $0F
   #LIBUSB_ENDPOINT_DIR_MASK = $80
   #LIBUSB_TRANSFER_TYPE_MASK = $03
   #LIBUSB_ISO_SYNC_TYPE_MASK = $0C
   #LIBUSB_ISO_USAGE_TYPE_MASK = $30

   #LIBUSB_CONTROL_SETUP_SIZE = SizeOf(uControlSetup)

   #LIBUSB_ERROR_COUNT = 14

   #LIBUSB_HOTPLUG_MATCH_ANY = -1

   ;-
   ;- -- Import LibUsb.
   Import "libusb-1.0.21.lib"
      libusb_bulk_transfer.i(*uDeviceHandle, iEndpoint.a, *Data, iLength.i, *iTransferred, iTimeout.i)
      libusb_claim_interface.i(*uDeviceHandle, iInterfaceNumber.i)
      libusb_clear_halt.i(*uDeviceHandle, iEndpoint.a)
      libusb_close(*uDeviceHandle)
      libusb_control_transfer.i(*uDeviceHandle, iRequestType.a, iRequest.a, iValue.u, iIndex.u, *Data, iLength.u, iTimeout.i)
      libusb_error_name.i(eErrorCode.i)
      libusb_exit(*uContext)
      libusb_free_config_descriptor(*uConfigDescriptor)
      libusb_free_device_list(*uDeviceList, iUnrefDevices.i)
      libusb_get_active_config_descriptor.i(*uDevice, *uConfigDescriptor)
      libusb_get_bus_number.a(*uDevice)
      libusb_get_config_descriptor(*uDevice, iIndex.a, *uConfigDescriptor)
      libusb_get_device.i(*uDeviceHandle)
      libusb_get_device_descriptor.i(*uDevice, *uDeviceDescriptor)
      libusb_get_device_list.i(*uContext, *uDeviceList)
      libusb_get_device_speed.i(*uDeviceHandle)
      libusb_get_port_number.a(*uDevice)
      libusb_get_string_descriptor_ascii.i(*uDeviceHandle, iIndex.a, *sBuffer, iBufferSize.i)
      libusb_get_version.i()
      libusb_has_capability(eCapability.l)
      libusb_init.i(*uContext)
      libusb_interrupt_transfer.i(*uDeviceHandle, iEndpoint.a, *Data, iLength.i, *iTransferred, iTimeout.i)
      libusb_open.i(*uDevice, *uDeviceHandle)
      libusb_release_interface.i(*uDeviceHandle, iInterfaceNumber.i)
      libusb_set_configuration.i(*uDeviceHandle, iConfiguration.i)
      libusb_set_debug(*uContext, eLevel.i)
      libusb_setlocale.i(*sLocale)
      libusb_strerror.i(eErrorCode.i)
   EndImport


   ;-
   ;- -- Declares.
   Declare.s ErrorName(iErrorCode.i)
   Declare.s StrError(iErrorCode.i)
   
   ;-

   Procedure.i BulkTransfer(*uDeviceHandle, iEndpoint.i, *Data, iLength.i, *iTransferred.Integer, iTimeout.i, *sError.String)
      ; Führt einen USB Bulk Transfer durch.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iEndpoint:
      ; <> *Data:          Der Speicher der Daten für das Senden und Empfangen.
      ; -> iLength:        Bei BulkWrites die Anzahl der Bytes die gesendet werden.
      ;                    Bei BulkReads die maximale Anzahl der Bytes die empfangen werden können.
      ; <- *iTransferred:  Die Anzahl der übertragenen Bytes.
      ;                    Wenn die Information nicht benötigt wird, kann hier Null übergeben werden.
      ; -> iTimeout:       Wartezeit in Millisekunden in der auf eine Antwort gewartet wird bevor abgebrochen wird.
      ;                    Für eine endlose Wartezeit ohne Abbruch kann 0 übergeben werden.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn der Transfer erfolgreich war, ansonst False.
      ; int libusb_bulk_transfer(struct libusb_device_handle *dev_handle, unsigned char endpoint,
      ;                          unsigned char *Data, int length, int *transferred, unsigned int timeout)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDeviceHandle = 0")
      ElseIf (Not *Data)
         *sError\s = CurrentProcedureName + "Der Zeiger auf die Daten ist Null. *Data = 0"
         DebugView::Error("Der Zeiger auf die Daten ist Null. *Data = 0")
      ElseIf (Not iLength)
         *sError\s = CurrentProcedureName + "Die Längenangabe der Daten ist Null. iLength = 0"
         DebugView::Error("Die Längenangabe der Daten ist Null. iLength = 0")

      Else
         iError = libusb_bulk_transfer(*uDeviceHandle, iEndpoint, *Data, iLength, *iTransferred, iTimeout)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i ClaimInterface(*uDeviceHandle, iInterfaceNumber.i, *sError.String)
      ; Reserviert die Schnittstelle.
      ; -> *uDeviceHandle:    Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iInterfaceNumber:  Die Nummer des Interface.
      ; <- sError:            Fehlerbeschreibung.
      ; Rückgabe:             True wenn die Schnittstelle reserviert wurde, ansonst False.
      ; int libusb_claim_interface(libusb_device_handle *dev_handle, int interface_number)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      Else
         iError = libusb_claim_interface(*uDeviceHandle, iInterfaceNumber)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i ClearHalt(*uDeviceHandle, iEndpoint.i, *sError.String)
      ; Löscht den Halt-Status des Endpoints vom übergebenen Gerät.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iEndpoint:      Die Nummer des Endpoints.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn der Halt-Status gelöscht wurde, ansonst False.
      ; int libusb_clear_halt (libusb_device_handle * dev, unsigned char endpoint)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      Else
         iError = libusb_clear_halt(*uDeviceHandle, iEndpoint)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure
   
   
   Procedure Close(*uDeviceHandle)
      ; Schließt das geöffnete Gerät mit dem übergebenen Handle.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; void libusb_close(libusb_device_handle *dev_handle)
      DebugView::ProcIn()

      If (Not *uDeviceHandle)
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      Else
         libusb_close(*uDeviceHandle)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure.i ControlTransfer(*uDeviceHandle, iRequestType.a, iRequest.a, iValue.u, iIndex.u, *Data, *iLength.Integer,
                               iTimeout.i, *sError.String)
      ; Führt einen USB Control Transfer durch.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iRequestType:   Der Request Type für das Setup Packet.
      ; -> iRequest:       Der Request für das Setup Packet.
      ; -> iValue:         Der Wert für das Setup Packet.
      ; -> iIndex:         Der Index für das Setup Packet.
      ; <> *Data:          Der Speicher der Daten für das Senden und Empfangen, je nach Request Type.
      ; <> *iLength:       Die Größe des Speichers für den Empfang der Daten.
      ;                    Die Größe der empfangenen Daten bei der Rückgabe.
      ; -> iTimeout:       Wartezeit in Millisekunden in der auf eine Antwort gewartet wird bevor abgebrochen wird.
      ;                    Für eine endlose Wartezeit ohne Abbruch kann 0 übergeben werden.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn der Transfer erfolgreich war, ansonst False.
      ; int libusb_control_transfer(libusb_device_handle *dev_handle, uint8_t bmRequestType, uint8_t bRequest, uint16_t wValue,
      ;                             uint16_t wIndex, unsigned char *Data, uint16_twLength, unsigned int timeout)
      DebugView::ProcIn()
      Protected fReturn.i, iResult.i, iLength.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      Else
         iLength = *iLength\i
         iResult = libusb_control_transfer(*uDeviceHandle, iRequestType, iRequest, iValue, iIndex, *Data, iLength, iTimeout)

         If Not (iResult > 0)
            *sError\s = CurrentProcedureName + ErrorDescription(iResult)
            DebugView::Error(ErrorDescription(iResult))

         Else
            fReturn = #True
            *iLength\i = iResult
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.s ErrorDescription(eErrorCode.i)
      ; Gibt eine Fehlerbeschreibung zurück.
      ; -> eErrorCode:  Fehlercode von Enumeration libusb_error.
      ; Rückgabe:       Die Fehlerbeschreibung.
      DebugView::ProcIn()
      Protected sReturn.s, sName.s, sDescription.s

      sName = ErrorName(eErrorCode)
      sDescription = StrError(eErrorCode)
      sReturn = "(" + Str(eErrorCode) + ") " + sName + " - " + sDescription

      DebugView::ProcOut()
      ProcedureReturn sReturn
   EndProcedure


   Procedure.s ErrorName(eErrorCode.i)
      ; Gibt den Fehlernamen zurück.
      ; -> eErrorCode:  Fehlercode von Enumeration libusb_error.
      ; Rückgabe:       Den Fehlernamen.
      ; const char* libusb_error_name (int error_code)
      DebugView::ProcIn()
      Protected sReturn.s, *sASCII

      *sASCII = libusb_error_name(eErrorCode)
      sReturn = PeekS(*sASCII, 100, #PB_Ascii)

      DebugView::ProcOut()
      ProcedureReturn sReturn
   EndProcedure


   Procedure FreeConfigDescriptor(*uConfigDescriptor)
      ; Gibt den Konfigurations-Descriptor wieder frei, der vorher mit GetActiveConfigDescriptor oder GetConfigDescriptor ermittelt.
      ; -> *uConfigDescriptor:   Der Zeiger auf die ConfigDesctiptor-Struktur.
      ; void libusb_free_config_descriptor(struct libusb_config_descriptor *config)
      DebugView::ProcIn()

      If (Not *uConfigDescriptor)
         DebugView::Error("Der Zeiger des Descriptors ist Null. *uConfigDescriptor = 0")

      Else
         libusb_free_config_descriptor(*uConfigDescriptor)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure Exit(*uContext)
      ; Deinitialisiert die libusb.
      ; Diese Prozedur sollte aufgerufen werden, wenn alle geöffneten Geräte geschlossen wurden
      ; und die libusb nicht mehr benötigt wird.
      ; -> *Context:    Der Zeiger zur Context-Struktur oder #Null für den Standard-Context.
      ; void libusb_exit (struct libusb_context *ctx)
      DebugView::ProcIn()

      libusb_exit(*uContext)

      DebugView::ProcOut()
   EndProcedure


   Procedure FreeDeviceList(*DeviceList, fUnrefDevices.i = #True)
      ; Gibt die Geräteliste frei, die vorher mit GetDeviceList angefordert wurde.
      ; -> *DeviceList:    Zeiger auf die Geräteliste.
      ; -> fUnrefDevices:  Bei True werden die Referenzzähler der einzelnen Geräte verringert.
      ; void libusb_free_device_list (libusb_device **list, int unref_devices)
      DebugView::ProcIn()

      If (Not *DeviceList)
         DebugView::Error("Der Zeiger der Geräteliste ist Null. *DeviceList = 0")

      Else
         libusb_free_device_list(*DeviceList, fUnrefDevices)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure.i GetActiveConfigDescriptor(*uDevice, *uConfigDescriptor, *sError.String)
      ; Ermittelt den Descriptor zur aktiven Konfiguration zum übergebenen Gerät.
      ; Der Descriptor muss mit FreeConfigDescriptor wieder freigegeben werden.
      ; -> *uDevice:             Der Zeiger auf die Gerätestruktur.
      ; <- *uConfigDescriptor:   Der Zeiger auf den Konfigurations-Descriptor.
      ; <- sError:               Fehlerbeschreibung.
      ; Rückgabe:                True wenn der Descriptor ermittelt wurde, ansonst False.
      ; int libusb_get_active_config_descriptor(libusb_device *dev, struct libusb_config_descriptor **config)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDevice = 0")
      ElseIf (Not *uConfigDescriptor)
         *sError\s = CurrentProcedureName + "Der Zeiger für den Descriptor ist Null. *uConfigDescriptor = 0"
         DebugView::Error("Der Zeiger für den Descriptor ist Null. *uConfigDescriptor = 0")

      Else
         iError = libusb_get_active_config_descriptor(*uDevice, *uConfigDescriptor)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetBusNumber(*uDevice, *iBusNumber.Integer, *sError.String)
      ; Gibt die Nummer von dem Bus zurück, an dem das Gerät verbunden ist.
      ; -> *uDevice:    Der Zeiger auf die Gerätestruktur.
      ; <- *iBusNumer:  Die Busnummer.
      ; <- sError:      Fehlerbeschreibung.
      ; Rückgabe:       True wenn die Busnummer ermittelt wurde, ansonst False.
      ; uint8_t libusb_get_bus_number (libusb_device *dev)
      DebugView::ProcIn()
      Protected fReturn.i, iNumber.a

      If (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDevice = 0")
      ElseIf (Not *iBusNumber)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Busnummer ist Null. *iBusNumber = 0"
         DebugView::Error("Der Zeiger für die Busnummer ist Null. *iBusNumber = 0")

      Else
         ; Den Rückgabewert direkt als Integer einlesen funktioniert nicht.
         iNumber = libusb_get_bus_number(*uDevice)
         *iBusNumber\i = iNumber
         fReturn = Bool(*iBusNumber\i)
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetConfigDescriptor(*uDevice, iIndex.i, *uConfigDescriptor, *sError.String)
      ; Ermittelt den Descriptor mit dem übergebenen Index.
      ; Der Descriptor muss mit FreeConfigDescriptor wieder freigegeben werden.
      ; -> *uDevice:             Der Zeiger auf die Gerätestruktur.
      ; -> iIndex:               Der Index der Konfiguration beginnend mit Null.
      ; <- *uConfigDescriptor:   Der Zeiger auf den Konfigurations-Descriptor.
      ; <- sError:               Fehlerbeschreibung.
      ; Rückgabe:                True wenn der Descriptor ermittelt wurde, ansonst False.
      ; int libusb_get_config_descriptor(libusb_device *dev, uint8_t config_index, struct libusb_config_descriptor **config)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i, iIndexNr.a

      If (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDevice = 0")
      ElseIf (Not *uConfigDescriptor)
         *sError\s = CurrentProcedureName + "Der Zeiger für den Descriptor ist Null. *uConfigDescriptor = 0"
         DebugView::Error("Der Zeiger für den Descriptor ist Null. *uConfigDescriptor = 0")

      Else
         iIndexNr = iIndex
         iError = libusb_get_config_descriptor(*uDevice, iIndexNr, *uConfigDescriptor)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetDevice(*uDeviceHandle, *uDevice.Integer, *sError.String)
      ; Gibt den Zeiger auf die Gerätestruktur' für das geöffnete Gerät zurück.
      ; -> *uDeviceHandle: Der Zeiger auf das geöffnete Gerät.
      ; <- *uDevice:       Der Zeiger auf die Gerätestruktur.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn die Gerätestruktur ermittelt wurde, ansonst False.
      ; libusb_device* libusb_get_device(libusb_device_handle *dev_handle)
      DebugView::ProcIn()
      Protected fReturn.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger auf das geöffnete Gerät ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger auf das geöffnete Gerät ist Null. *uDeviceHandle = 0")
      ElseIf (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Gerätestruktur ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger für die Gerätestruktur ist Null. *uDevice = 0")

      Else
         *uDevice\i = libusb_get_device(*uDeviceHandle)
         fReturn = Bool(*uDevice\i)
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetDeviceDescriptor(*uDevice, *uDeviceDescriptor, *sError.String)
      ; Gibt den Zeiger zu dem Descriptor zurück.
      ; -> *uDevice:             Der Zeiger auf die Gerätestruktur.
      ; <- *uDeviceDescriptor:   Der Zeiger auf den Descriptor.
      ; <- sError:               Fehlerbeschreibung.
      ; Rückgabe:                True wenn der Descriptor ermittelt wurde, ansonst False.
      ; int libusb_get_device_descriptor(libusb_device *dev, struct libusb_device_descriptor *desc)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDevice = 0")
      ElseIf (Not *uDeviceDescriptor)
         *sError\s = CurrentProcedureName + "Der Zeiger für den Descriptor ist Null. *uDeviceDescriptor = 0"
         DebugView::Error("Der Zeiger für den Descriptor ist Null. *uDeviceDescriptor = 0")

      Else
         iError = libusb_get_device_descriptor(*uDevice, *uDeviceDescriptor)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetDeviceList(*uContext, List *DeviceList.uDevice(), *iDeviceCount.Integer, *uFreeDeviceList.Integer, *sError.String)
      ; Gibt eine Liste zurück, die alle aktuell vorhandenen USB-Geräte enthält.
      ; -> *uContext:         Der Zeiger zum Context oder #Null für den Standard-Context.
      ; <- *DeviceList():     Die Liste enthält die Zeiger zu den Gerätestrukturen.
      ; <- *iDeviceCount:     Die Anzahl der Geräte.
      ; <- *FreeDeviceList:   Der Zeiger zu der Geräteliste die wieder freigegeben werden muss.
      ; <- sError:            Fehlerbeschreibung.
      ; Rückgabe:             True wenn die Liste ermittelt wurde, ansonst False.
      ; ssize_t libusb_get_device_list (libusb_context *ctx, libusb_device ***list)
      DebugView::ProcIn()
      Protected fReturn.i, iResult.i, c.i, *uDeviceList.uDeviceList

      If (Not *iDeviceCount)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Geräteanzahl ist Null. *iDeviceCount = 0"
         DebugView::Error("Der Zeiger für die Geräteanzahl ist Null. *iDeviceCount = 0")
      ElseIf (Not *uFreeDeviceList)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Geräteliste ist Null. *uFreeDeviceList = 0"
         DebugView::Error("Der Zeiger für die Geräteliste ist Null. *uFreeDeviceList = 0")

      Else
         iResult = libusb_get_device_list(*uContext, @*uDeviceList)

         ; Fehler.
         If (iResult < 1)
          *iDeviceCount\i = 0
          *sError\s = CurrentProcedureName + ErrorDescription(iResult)
          DebugView::Error(ErrorDescription(iResult))

         Else
            ; Erfolg und Anzahl der Geräte zurückgeben.
            fReturn = #True
            *iDeviceCount\i = iResult
            *uFreeDeviceList\i = *uDeviceList

            ; Die Gerätezeiger in die Liste kopieren.
            ClearList(*DeviceList())
            For c = 0 To (*iDeviceCount\i - 1)
               AddElement(*DeviceList())
               *DeviceList() = *uDeviceList\uDevice[c]
            Next c
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetDeviceSpeed(*uDevice, *eSpeed.Integer, *sError.String)
      ; Gibt die USB-Geschwindigkeit zurück.
      ; -> *uDevice: Der Zeiger auf die Gerätestruktur.
      ; <- *eSpeed:  Die Geschwindigkeit als Enumeration libusb_speed.
      ; <- sError:   Fehlerbeschreibung.
      ; Rückgabe:    True wenn die Geschwindigkeit ermittelt wurde, ansonst False.
      ; uint8_t int libusb_get_device_speed(libusb_device *dev)
      DebugView::ProcIn()
      Protected fReturn.i

      If (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDevice = 0")
      ElseIf (Not *eSpeed)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Geschwindigkeit ist Null. *eSpeed = 0"
         DebugView::Error("Der Zeiger für die Geschwindigkeit ist Null. *eSpeed = 0")

      Else
         *eSpeed\i = libusb_get_device_speed(*uDevice)
         fReturn = #True
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetPortNumber(*uDevice, *iPortNumber.Integer, *sError.String)
      ; Gibt die Nummer von dem Port zurück, an dem das Gerät verbunden ist.
      ; -> *uDevice: Der Zeiger auf die Gerätestruktur.
      ; <- *iBusNumer:  Die Busnummer.
      ; <- sError:      Fehlerbeschreibung.
      ; Rückgabe:       True wenn die Portnummer ermittelt wurde, ansonst False.
      ; uint8_t libusb_get_port_number(libusb_device *dev)
      DebugView::ProcIn()
      Protected fReturn.i, iNumber.a


      If (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDevice = 0")
      ElseIf (Not *iPortNumber)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Portnummer ist Null. *iPortNumber = 0"
         DebugView::Error("Der Zeiger für die Portnummer ist Null. *iPortNumber = 0")

      Else
         ; Den Rückgabewert direkt als Integer einlesen funktioniert nicht.
         iNumber = libusb_get_bus_number(*uDevice)
         *iPortNumber\i = iNumber
         fReturn = Bool(*iPortNumber\i)
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure
   
   
   Procedure.i GetStringLanguageList(*uDeviceHandle, List LanguageIdList.i(), *sError.String)
      ; Ermittelt die IDs der unterstützten Sprachen für die String Descriptoren vom übergebenen Gerät.
      ; Rückgabe: True bei Erfolg, ansonst False.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; <- LanguageIdList: Die Liste für die Sprach-IDs.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn die IDs ermittelt wurde, ansonst False.
      DebugView::ProcIn()
      Protected fReturn.i, fResult.i, iResult.i, *uBuffer, iValue.u, iIndex.u
      Protected c.i, iAvailableIds.i
      Protected *uStringDescriptor.uStringDescriptor
      Protected iTimeout.i = 1000   ; Millisekunden.
      Protected iBufferSize.u = 200   ; Buffer für 100 Sprach-IDs.

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      Else
         *uBuffer = AllocateMemory(iBufferSize)
         If *uBuffer
            ; Für die Sprach-IDs wird der Language String Descripter mit Index Null und ohne Sprachauwahl abgefragt.
            iValue = ((#eLIBUSB_DT_STRING << 8) | #Null) ; Language String Descripter mit Index Null.
            iIndex = #Null ; Keine Sprachauwahl.                            ;
            iResult = libusb_control_transfer(*uDeviceHandle, #eLIBUSB_ENDPOINT_IN, #eLIBUSB_REQUEST_GET_DESCRIPTOR,
                                              iValue, iIndex, *uBuffer, iBufferSize, iTimeout)
            *uStringDescriptor = *uBuffer
            If Not (iResult > 0)
               *sError\s = CurrentProcedureName + ErrorDescription(iResult)
               DebugView::Error(ErrorDescription(iResult))
               
            ElseIf (iResult < 4)
               ; Der empfangene Descriptor muss mindestens vier Bytes lang sein und eine Sprache (Englisch, US) enthalten.
               *sError\s = CurrentProcedureName + ErrorDescription(#eLIBUSB_ERROR_IO)
               DebugView::Error(ErrorDescription(iResult))
               
            ElseIf Not (*uStringDescriptor\iDescriptorType = #eLIBUSB_DT_STRING)
               ; Es wurde kein String Descriptor empfangen.
               *sError\s = CurrentProcedureName + ErrorDescription(#eLIBUSB_ERROR_IO)
               DebugView::Error(ErrorDescription(iResult))
               
            ElseIf (*uStringDescriptor\iLength > iResult)
               ; Es wurde kein gültiger String Descriptor empfangen.
               *sError\s = CurrentProcedureName + ErrorDescription(#eLIBUSB_ERROR_IO)
               DebugView::Error(ErrorDescription(iResult))
               
            Else
               ; Der Language String Descriptor mit den unterstützten Sprachen wurde erfolgreich empfangen.
               fReturn = #True
               ; Die übergebene Liste mit den IDs füllen.
               ClearList(LanguageIdList())
               iAvailableIds = (*uStringDescriptor\iLength - OffsetOf(uStringDescriptor\iCountryCode))
               iAvailableIds / SizeOf(Unicode)  ; ACHTUNG: SizeOf(uStringDescriptor\iCountryCode) ergibt #NULL,
                                                ; wegen der Strukturdefinition iCountryCode[0], darum den Datentyp verwenden.
               iAvailableIds - 1
               For c = 0 To iAvailableIds
                  AddElement(LanguageIdList())
                  LanguageIdList() = *uStringDescriptor\iCountryCode[c]
               Next c   
            EndIf
            FreeMemory(*uBuffer)
         EndIf
      EndIf
            
      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure
   
   
   Procedure.i GetStringDescriptor(*uDeviceHandle, iStringIndex.i, iLanguageId.i, *sString.String, *sError.String)
      ; Ermittelt den Beschreibung des Descriptors vom übergebenen Gerät in der angegebenen Sprache.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iStringIndex:   Der Index des Descriptors.
      ; -> iLanguageId:    Die Id der Sprache. Wird #Null übergeben, wird die erste verfügbare Sprache verwendet.
      ; <- sString:        Die Beschreibung des Descriptors.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn die Beschreibung ermittelt wurde, ansonst False.
      DebugView::ProcIn()
      Protected fReturn.i, fResult.i, iResult.i, *uBuffer, iValue.u, iIndex.u
      Protected *uStringDescriptor.uStringDescriptor
      Protected iTimeout.i = 1000   ; Millisekunden.
      Protected iBufferSize.u = 255 ; /* Some devices choke on size > 255 */
      NewList LanguageIdList.i()

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      ElseIf (Not *sString)
         *sError\s = CurrentProcedureName + "Der Zeiger für den StringDescriptor ist Null. *sString = 0"
         DebugView::Error("Der Zeiger für den StringDescriptor ist Null. *sString = 0")

      Else
         *uBuffer = AllocateMemory(iBufferSize)
         If *uBuffer
            
            ; Wenn keine Sprache angegeben wurde, die vorhandenen Sprachen ermitteln, die das Gerät anbietet.
            fResult = Bool(iLanguageId)
            If (Not fResult)
               fResult = GetStringLanguageList(*uDeviceHandle, LanguageIdList(), *sError)
               If fResult
                  ; Der Language String Descriptor mit den unterstützten Sprachen wurde erfolgreich empfangen.
                  fResult = Bool(FirstElement(LanguageIdList()))
                  If (Not fResult)
                     DebugView::Error("Die Liste LanguageIdList() mit den Ländercodes ist leer.")
                  Else
                     iLanguageId = LanguageIdList()
                     FreeList(LanguageIdList())
                   EndIf
               EndIf
            EndIf
         
            ; Den String Descriptor mit der angegebenen oder ersten vorhandenen Sprache anfordern.
            If fResult
               *uStringDescriptor = *uBuffer
               iValue = ((#eLIBUSB_DT_STRING << 8) | iStringIndex) ; Language String Descripter mit übergebenen StringIndex.
               iIndex = iLanguageId ; Die übergebene oder erste verfügbare Sprache verwenden.
               iResult = libusb_control_transfer(*uDeviceHandle, #eLIBUSB_ENDPOINT_IN, #eLIBUSB_REQUEST_GET_DESCRIPTOR,
                                                 iValue, iIndex, *uBuffer, iBufferSize, iTimeout)
               If Not (iResult > 0)
                  *sError\s = CurrentProcedureName + ErrorDescription(iResult)
                  DebugView::Error(ErrorDescription(iResult))
               ElseIf Not (*uStringDescriptor\iDescriptorType = #eLIBUSB_DT_STRING)
                  ; Es wurde kein String Descriptor empfangen.
                  *sError\s = CurrentProcedureName + ErrorDescription(#eLIBUSB_ERROR_IO)
                  DebugView::Error(ErrorDescription(iResult))
               ElseIf (*uStringDescriptor\iLength > iResult)
                  ; Es wurde kein gültiger String Descriptor empfangen.
                  *sError\s = CurrentProcedureName + ErrorDescription(#eLIBUSB_ERROR_IO)
                  DebugView::Error(ErrorDescription(iResult))
                  
               Else
                  ; Den String Descriptor zurückgeben.
                  *sString\s = PeekS(@*uStringDescriptor\uChars[0], 
                                     *uStringDescriptor\iLength - OffsetOf(uStringDescriptor\uChars), #PB_Unicode) 
                  fReturn = #True
               EndIf
            EndIf
            
            FreeMemory(*uBuffer)
            
         EndIf
      EndIf
      
      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetStringDescriptorAscii(*uDeviceHandle, iStringIndex.i, *sString.String, *sError.String)
      ; Ermittelt den Beschreibung des Descriptors in der ersten verfügbaren Sprache die das Gerät anbietet.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iStringIndex:   Der Index des Descriptors.
      ; <- sString:        Die Beschreibung des Descriptors.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn die Beschreibung ermittelt wurde, ansonst False.
      ; int libusb_get_string_descriptor_ascii(libusb_device_handle *dev_handle, int8_t desc_index, unsigned char *data, int length)
      DebugView::ProcIn()
      Protected fReturn.i, iResult.i, *sBuffer
      #iBufferSize = 255

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      ElseIf (Not *sString)
         *sError\s = CurrentProcedureName + "Der Zeiger für den StringDescriptor ist Null. *sString = 0"
         DebugView::Error("Der Zeiger für den StringDescriptor ist Null. *sString = 0")

      Else
         *sBuffer = AllocateMemory(#iBufferSize)
         If *sBuffer
            iResult = libusb_get_string_descriptor_ascii(*uDeviceHandle, iStringIndex, *sBuffer, #iBufferSize)

            If Not (iResult > 0)
               *sError\s = CurrentProcedureName + ErrorDescription(iResult)
               DebugView::Error(ErrorDescription(iResult))

            Else
               fReturn = #True
               *sString\s = Trim(PeekS(*sBuffer, iResult, #PB_Ascii))
            EndIf

            FreeMemory(*sBuffer)
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i GetVersion(*sVersion.String, *sDescription.String, *sError.String)
      ; Ermittelt die Versionsnummer und Beschreibung der LibUsb.
      ; <- sVersion:       Versionsnummer als Zeichenfolge, z.B.: "1.0.15.10646-rc4".
      ; <- sDescription:   Beschreibung, z.B.: "http://libusb.info".
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn die Versionsnummer ermittelt wurde, ansonst False.
      ; const struct libusb_version * libusb_get_version(void)
      DebugView::ProcIn()
      Protected *uVersion.uVersion
      Protected fReturn.i, iChars.i

      If (Not *sVersion)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Version ist Null. *sVersion = 0"
         DebugView::Error("Der Zeiger für die Version ist Null. *sVersion = 0")

      ElseIf (Not *sDescription)
         *sError\s = CurrentProcedureName + "Der Zeiger für die Beschreibung ist Null. *sDescription = 0"
         DebugView::Error("Der Zeiger für die Beschreibung ist Null. *sDescription = 0")

      Else
         *uVersion = libusb_get_version()
         fReturn = Bool(*uVersion)

         If (Not fReturn)
            *sVersion\s = #Null$
            *sDescription\s = #Null$
            *sError\s = CurrentProcedureName + "Es wurde ein Null-Zeiger zurückgegeben."
            DebugView::Error("Es wurde ein Null-Zeiger zurückgegeben.")
         EndIf

         If fReturn
            With *uVersion
               *sVersion\s = StrU(\iMajor, #PB_Word) +
                             "." + StrU(\iMinor, #PB_Word) +
                             "." + StrU(\iMicro, #PB_Word) +
                             "." + StrU(\iNano, #PB_Word)
               If \aReleaseCandidate
                  iChars = MemoryStringLength(\aReleaseCandidate, #PB_Ascii)
                  If iChars
                     If (iChars > 32)
                        iChars = 32
                     EndIf
                     *sVersion\s + PeekS(\aReleaseCandidate, iChars, #PB_Ascii)
                  EndIf
               EndIf
               If \aDescribe
                  iChars = MemoryStringLength(\aDescribe, #PB_Ascii)
                  If iChars
                     If (iChars > 64)
                        iChars = 64
                     EndIf
                     *sDescription\s = PeekS(\aDescribe, iChars, #PB_Ascii)
                  EndIf
               EndIf
            EndWith
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i HasCapability(eCapability.i)
      ; Ermittelt ob die LibUSB bestimmte Fähigkeiten besitzt.
      ; -> eCapability: Die Fähigkeit die abgefragt wird.
      ; Rückgabe:       True wenn die Fähigkeit vorhanden ist, ansonst False.
      ; int libusb_has_capability(uint32_t capability)
      DebugView::ProcIn()
      Protected fReturn.i, iCapability.l

      iCapability = eCapability
      fReturn = Bool(libusb_has_capability(iCapability))

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i Init(*uContext, *sError.String)
      ; Initialisiert die libusb.
      ; Diese Prozedur muss vor allen anderen LibUsb-Prozeduren aufgerufen werden.
      ; Wenn kein Zeiger bzw. #Null für eine Context-Struktur übergeben wird, wird ein Standard-Context erstellt.
      ; Ist schon ein Standard-Context vorhanden, wird dieser wiederverwendet, es wird nichts neu- oder reinitialisiert.
      ; -> *uContext:   Zeiger auf die Context-Struktur oder #Null.
      ; <- sError:      Fehlerbeschreibung.
      ; Rückgabe:       True wenn die libusb initialisiert wurde, ansonst False.
      ; int libusb_init(libusb_context **context)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      iError = libusb_init(*uContext)

      If iError
         *sError\s = CurrentProcedureName + ErrorDescription(iError)
         DebugView::Error(ErrorDescription(iError))

      Else
         fReturn = #True
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i InterruptTransfer(*uDeviceHandle, iEndpoint.i, *Data, iLength.i, *iTransferred.Integer, iTimeout.i, *sError.String)
      ; Führt einen USB Interrupt Transfer durch.
      ; -> *uDeviceHandle: Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iEndpoint:
      ; <> *Data:          Der Speicher der Daten für das Senden und Empfangen.
      ; -> iLength:        Bei BulkWrites die Anzahl der Bytes die gesendet werden.
      ;                    Bei BulkReads die maximale Anzahl der Bytes die empfangen werden können.
      ; <- *iTransferred:  Die Anzahl der übertragenen Bytes.
      ;                    Wenn die Information nicht benötigt wird, kann hier Null übergeben werden.
      ; -> iTimeout:       Wartezeit in Millisekunden in der auf eine Antwort gewartet wird bevor abgebrochen wird.
      ;                    Für eine endlose Wartezeit ohne Abbruch kann 0 übergeben werden.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn der Transfer erfolgreich war, ansonst False.
      ; int libusb_bulk_transfer(struct libusb_device_handle *dev_handle, unsigned char endpoint,
      ;                          unsigned char *Data, int length, int *transferred, unsigned int timeout)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDeviceHandle = 0")
      ElseIf (Not *Data)
         *sError\s = CurrentProcedureName + "Der Zeiger auf die Daten ist Null. *Data = 0"
         DebugView::Error("Der Zeiger auf die Daten ist Null. *Data = 0")
      ElseIf (Not iLength)
         *sError\s = CurrentProcedureName + "Die Längenangabe der Daten ist Null. iLength = 0"
         DebugView::Error("Die Längenangabe der Daten ist Null. iLength = 0")

      Else
         iError = libusb_interrupt_transfer(*uDeviceHandle, iEndpoint, *Data, iLength, *iTransferred, iTimeout)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i Open(*uDevice, *uDeviceHandle, *sError.String)
      ; Öffnet ein Gerät und gibt ein Handle für den Zugriff darauf zurück.
      ; -> *uDevice:       Der Zeiger auf die Gerätestruktur die mit GetDeviceList ermittelt wurde.
      ; <- *uDeviceHandle: Das Handle bzw. Zeiger auf das geöffnete Gerät.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn das Gerät geöffnet wurde, ansonst False.
      ; int libusb_open(libusb_device *dev, libusb_device_handle **dev_handle)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDevice)
         *sError\s = CurrentProcedureName + "Der Zeiger des Geräts ist Null. *uDevice = 0"
         DebugView::Error("Der Zeiger des Geräts ist Null. *uDevice = 0")
      ElseIf (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger für das geöffnete Gerät ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger für das geöffnete Gerät ist Null. *uDeviceHandle = 0")

      Else
         iError = libusb_open(*uDevice, *uDeviceHandle)

         If iError
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i ReleaseInterface(*uDeviceHandle, iInterfaceNumber.i, *sError.String)
      ; Gibt die reserviert die Schnittstelle wieder frei.
      ; -> *uDeviceHandle:    Das Handle vom Gerät das mit der Funktion Open erhalten wurde.
      ; -> iInterfaceNumber:  Die Nummer des Interface.
      ; <- sError:            Fehlerbeschreibung.
      ; Rückgabe:             True wenn die Schnittstelle freigegeben wurde, ansonst False.
      ; int libusb_release_interface(libusb_device_handle *dev_handle, int interface_number)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      Else
         iError = libusb_release_interface(*uDeviceHandle, iInterfaceNumber)

         If Not (iError = #eLIBUSB_SUCCESS)
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.i SetConfiguration(*uDeviceHandle, iConfiguration.i, *sError.String)
      ; Setzt die übergebene Konfiguration des Geräts als aktiv.
      ; -> *uDeviceHandle: Das Handle bzw. Zeiger auf das geöffnete Gerät.
      ; -> iConfiguration: Der Index der Konfiguration, -1 für das Setzen des unkonfigurierten Zustands.
      ; <- sError:         Fehlerbeschreibung.
      ; Rückgabe:          True wenn das Gerät konfiguriert wurde, ansonst False.
      ; int libusb_set_configuration(libusb_device_handle *dev_handle, int configuration)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i

      If (Not *uDeviceHandle)
         *sError\s = CurrentProcedureName + "Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0"
         DebugView::Error("Der Zeiger des geöffneten Geräts ist Null. *uDeviceHandle = 0")

      Else
         iError = libusb_set_configuration(*uDeviceHandle, iConfiguration)

         If iError
            *sError\s = CurrentProcedureName + ErrorDescription(iError)
            DebugView::Error(ErrorDescription(iError))

         Else
            fReturn = #True
         EndIf
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure SetDebug(*uContext, eLevel.i)
      ; Setzt den Debug-Level für die Info- Warn- und Fehlermeldungen, die an StdOut und SdtErr ausgegeben werden.
      ; -> *Context: Der Zeiger zur Context-Struktur oder #Null für den Standard-Context.
      ; -> eLevel:   Ein Log-Level von der Enumeration #eLIBUSB_LOG_LEVEL_*.
      ; void libusb_set_debug(libusb_context *ctx, int level)
      DebugView::ProcIn()

      libusb_set_debug(*uContext, eLevel)

      DebugView::ProcOut()
   EndProcedure


   Procedure.i SetLocale(sLocale.s, *sError.String)
      ; Wählt die Sprache aus, in der die Fehlermeldungen ausgegeben werden.
      ; Aktuell wird unterstützt: englisch (en), niederländisch (nl), französisch (fr) und russisch (ru).
      ; -> sLocale:  Der aus 2 Buchstaben bestehende ISO 639-1 Sprachcode, z.B. en, nl, fr, ru.
      ; <- sError:   Fehlerbeschreibung.
      ; Rückgabe:    True wenn die Sprache ausgewählt wurde, ansonst False.
      ; int libusb_setlocale(const char *locale)
      DebugView::ProcIn()
      Protected fReturn.i, iError.i, *sAscii

      *sAscii = Ascii(sLocale)
      iError = libusb_setlocale(*sAscii)
      FreeMemory(*sAscii)

      If iError
         *sError\s = CurrentProcedureName + ErrorDescription(iError) + " - sLocale = " + sLocale
         DebugView::Error(ErrorDescription(iError) + " - sLocale = " + sLocale)

      Else
         fReturn = #True
      EndIf

      DebugView::ProcOut()
      ProcedureReturn fReturn
   EndProcedure


   Procedure.s StrError(eErrorCode.i)
      ; Gibt eine kurze Beschreibung des Fehlers in der per SetLocale ausgewählten Sprache zurück.
      ; -> eErrorCode:  Fehlercode von Enumeration libusb_error.
      ; Rückgabe:       Die Fehlerbeschreibung.
      ; const char* libusb_strerror(enum libusb_error errcode)
      DebugView::ProcIn()
      Protected sReturn.s, *sUTF8

      *sUTF8 = libusb_strerror(eErrorCode)
      sReturn = PeekS(*sUTF8, 200, #PB_UTF8)

      DebugView::ProcOut()
      ProcedureReturn sReturn
   EndProcedure


EndModule   ; LibUsb.